@@ -8,15 +8,13 @@ package IO::Async::Function;
 use strict;
 use warnings;
 
-our $VERSION = '0.67';
+our $VERSION = '0.70';
 
 use base qw( IO::Async::Notifier );
 use IO::Async::Timer::Countdown;
 
 use Carp;
 
-use Storable qw( freeze );
-
 =head1 NAME
 
 C<IO::Async::Function> - call a function asynchronously
@@ -72,7 +70,7 @@ all of the result must be represented in the return values.
 The Function object is implemented using an L<IO::Async::Routine> with two
 L<IO::Async::Channel> objects to pass calls into and results out from it.
 
-The C<IO::Async> framework generally provides mechanisms for multiplexing IO
+The L<IO::Async> framework generally provides mechanisms for multiplexing IO
 tasks between different handles, so there aren't many occasions when such an
 asynchronous function is necessary. Two cases where this does become useful
 are:
@@ -88,7 +86,7 @@ When a large amount of computationally-intensive work needs to be performed
 
 When a blocking OS syscall or library-level function needs to be called, and
 no nonblocking or asynchronous version is supplied. This is used by
-C<IO::Async::Resolver>.
+L<IO::Async::Resolver>.
 
 =back
 
@@ -107,9 +105,19 @@ The following named parameters may be passed to C<new> or C<configure>:
 
 The body of the function to execute.
 
+ @result = $code->( @args )
+
+=head2 init_code => CODE
+
+Optional. If defined, this is invoked exactly once in every child process or
+thread, after it is created, but before the first invocation of the function
+body itself.
+
+ $init_code->()
+
 =head2 model => "fork" | "thread"
 
-Optional. Requests a specific C<IO::Async::Routine> model. If not supplied,
+Optional. Requests a specific L<IO::Async::Routine> model. If not supplied,
 leaves the default choice up to Routine.
 
 =head2 min_workers => INT
@@ -215,7 +223,7 @@ sub configure
 
    my $need_restart;
 
-   foreach (qw( code setup )) {
+   foreach (qw( init_code code setup )) {
       $need_restart++, $self->{$_} = delete $params{$_} if exists $params{$_};
    }
 
@@ -251,7 +259,9 @@ L<Future> instances.
 
 =cut
 
-=head2 $function->start
+=head2 start
+
+   $function->start
 
 Start the worker processes
 
@@ -264,7 +274,9 @@ sub start
    $self->_new_worker for 1 .. $self->{min_workers};
 }
 
-=head2 $function->stop
+=head2 stop
+
+   $function->stop
 
 Stop the worker processes
 
@@ -280,7 +292,9 @@ sub stop
    }
 }
 
-=head2 $function->restart
+=head2 restar
+
+   $function->restart
 
 Gracefully stop and restart all the worker processes. 
 
@@ -294,7 +308,9 @@ sub restart
    $self->start;
 }
 
-=head2 @result = $function->call( %params )->get
+=head2 call
+
+   @result = $function->call( %params )->get
 
 Schedules an invocation of the contained function to be executed on one of the
 worker processes. If a non-busy worker is available now, it will be called
@@ -315,7 +331,21 @@ A reference to the array of arguments to pass to the code.
 
 =back
 
-=head2 $function->call( %params )
+If the function body returns normally the list of results are provided as the
+(successful) result of returned future. If the function throws an exception
+this results in a failed future. In the special case that the exception is in
+fact an unblessed C<ARRAY> reference, this array is unpacked and used as-is
+for the C<fail> result. If the exception is not such a reference, it is used
+as the first argument to C<fail>, in the category of C<error>.
+
+   $f->done( @result )
+
+   $f->fail( @{ $exception } )
+   $f->fail( $exception, error => )
+
+=head2 call (void)
+
+   $function->call( %params )
 
 When not returning a future, the C<on_result>, C<on_return> and C<on_error>
 arguments give continuations to handle successful results or failure.
@@ -393,7 +423,7 @@ sub call
       croak "Expected either 'on_result' or 'on_return' and 'on_error' keys, or to return a Future";
    }
 
-   my $request = freeze( $args );
+   my $request = IO::Async::Channel->encode( $args );
 
    my $future;
    if( my $worker = $self->_get_worker ) {
@@ -426,7 +456,9 @@ sub _worker_objects
    return values %{ $self->{workers} };
 }
 
-=head2 $count = $function->workers
+=head2 workers
+
+   $count = $function->workers
 
 Returns the total number of worker processes available
 
@@ -438,7 +470,9 @@ sub workers
    return scalar keys %{ $self->{workers} };
 }
 
-=head2 $count = $function->workers_busy
+=head2 workers_busy
+
+   $count = $function->workers_busy
 
 Returns the number of worker processes that are currently busy
 
@@ -450,7 +484,9 @@ sub workers_busy
    return scalar grep { $_->{busy} } $self->_worker_objects;
 }
 
-=head2 $count = $function->workers_idle
+=head2 workers_idle
+
+   $count = $function->workers_idle
 
 Returns the number of worker processes that are currently idle
 
@@ -467,7 +503,7 @@ sub _new_worker
    my $self = shift;
 
    my $worker = IO::Async::Function::Worker->new(
-      ( map { $_ => $self->{$_} } qw( model code setup exit_on_die ) ),
+      ( map { $_ => $self->{$_} } qw( model init_code code setup exit_on_die ) ),
       max_calls => $self->{max_worker_calls},
 
       on_finish => $self->_capture_weakself( sub {
@@ -550,8 +586,11 @@ sub new
    my $arg_channel = IO::Async::Channel->new;
    my $ret_channel = IO::Async::Channel->new;
 
+   my $init = delete $params{init_code};
    my $code = delete $params{code};
    $params{code} = sub {
+      $init->() if defined $init;
+
       while( my $args = $arg_channel->recv ) {
          my @ret;
          my $ok = eval { @ret = $code->( @$args ); 1 };
@@ -559,6 +598,10 @@ sub new
          if( $ok ) {
             $ret_channel->send( [ r => @ret ] );
          }
+         elsif( ref $@ ) {
+            # Presume that $@ is an ARRAYref of error results
+            $ret_channel->send( [ e => @{ $@ } ] );
+         }
          else {
             chomp( my $e = "$@" );
             $ret_channel->send( [ e => $e, error => ] );
@@ -610,7 +653,7 @@ sub call
    my $worker = shift;
    my ( $args ) = @_;
 
-   $worker->{arg_channel}->send_frozen( $args );
+   $worker->{arg_channel}->send_encoded( $args );
 
    $worker->{busy} = 1;
    $worker->{max_calls}--;
@@ -653,6 +696,23 @@ sub call
    }));
 }
 
+=head1 EXAMPLES
+
+=head2 Extended Error Information on Failure
+
+The array-unpacking form of exception indiciation allows the function body to
+more precicely control the resulting failure from the C<call> future.
+
+ my $divider = IO::Async::Function->new(
+    code => sub {
+       my ( $numerator, $divisor ) = @_;
+       $divisor == 0 and
+          die [ "Cannot divide by zero", div_zero => $numerator, $divisor ];
+
+       return $numerator / $divisor;
+    }
+ );
+
 =head1 NOTES
 
 For the record, 123454321 is 11111 * 11111, a square number, and therefore not
