@@ -8,7 +8,7 @@ package IO::Async::Protocol::LineStream;
 use strict;
 use warnings;
 
-our $VERSION = '0.61';
+our $VERSION = '0.70';
 
 use base qw( IO::Async::Protocol::Stream );
 
@@ -72,14 +72,10 @@ Invoked when a new complete line of input is received.
 
 The following named parameters may be passed to C<new> or C<configure>:
 
-=over 8
-
-=item on_read_line => CODE
+=head2 on_read_line => CODE
 
 CODE reference for the C<on_read_line> event.
 
-=back
-
 =cut
 
 sub _init
@@ -118,7 +114,9 @@ sub on_read
 
 =cut
 
-=head2 $lineprotocol->write_line( $text )
+=head2 write_line
+
+   $lineprotocol->write_line( $text )
 
 Writes a line of text to the transport stream. The text will have the
 end-of-line marker appended to it; C<$text> should not end with it.
