@@ -1,15 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2006-2014 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2006-2015 -- leonerd@leonerd.org.uk
 
 package IO::Async::Stream;
 
 use strict;
 use warnings;
-use 5.010; # //
 
-our $VERSION = '0.64';
+our $VERSION = '0.70';
 
 use base qw( IO::Async::Handle );
 
@@ -20,6 +19,8 @@ use Carp;
 use Encode 2.11 qw( find_encoding STOP_AT_PARTIAL );
 use Scalar::Util qw( blessed );
 
+use IO::Async::Debug;
+
 # Tuneable from outside
 # Not yet documented
 our $READLEN  = 8192;
@@ -254,7 +255,7 @@ filehandle is write-ready. This is useful, for example, on streams that should
 contain up-to-date logging or console information.
 
 It currently defaults to false for any file handle, but future versions of
-C<IO::Async> may enable this by default on STDOUT and STDERR.
+L<IO::Async> may enable this by default on STDOUT and STDERR.
 
 =head2 read_len => INT
 
@@ -387,8 +388,11 @@ sub configure
    }
 
    if( exists $params{read_high_watermark} or exists $params{read_low_watermark} ) {
-      my $high = delete $params{read_high_watermark} // $self->{read_high_watermark};
-      my $low  = delete $params{read_low_watermark}  // $self->{read_low_watermark};
+      my $high = delete $params{read_high_watermark};
+      defined $high or $high = $self->{read_high_watermark};
+
+      my $low  = delete $params{read_low_watermark};
+      defined $low  or $low  = $self->{read_low_watermark};
 
       croak "Cannot set read_low_watermark without read_high_watermark" if defined $low and !defined $high;
       croak "Cannot set read_high_watermark without read_low_watermark" if defined $high and !defined $low;
@@ -439,9 +443,13 @@ L<Future> instances.
 
 =cut
 
-=head2 $stream->want_readready_for_read( $set )
+=head2 want_readready_for_read
 
-=head2 $stream->want_readready_for_write( $set )
+=head2 want_readready_for_write
+
+   $stream->want_readready_for_read( $set )
+
+   $stream->want_readready_for_write( $set )
 
 Mutators for the C<want_readready> property on L<IO::Async::Handle>, which
 control whether the C<read> or C<write> behaviour should be continued once the
@@ -474,9 +482,13 @@ sub want_readready_for_write
    $self->want_readready( $self->{want} & WANT_ANY_READ ) if $self->read_handle;
 }
 
-=head2 $stream->want_writeready_for_write( $set )
+=head2 want_writeready_for_read
 
-=head2 $stream->want_writeready_for_read( $set )
+=head2 want_writeready_for_write
+
+   $stream->want_writeready_for_write( $set )
+
+   $stream->want_writeready_for_read( $set )
 
 Mutators for the C<want_writeready> property on L<IO::Async::Handle>, which
 control whether the C<write> or C<read> behaviour should be continued once the
@@ -525,11 +537,13 @@ sub _is_empty
    return !@{ $self->{writequeue} };
 }
 
-=head2 $stream->close
+=head2 close
+
+   $stream->close
 
 A synonym for C<close_when_empty>. This should not be used when the deferred
 wait behaviour is required, as the behaviour of C<close> may change in a
-future version of C<IO::Async>. Instead, call C<close_when_empty> directly.
+future version of L<IO::Async>. Instead, call C<close_when_empty> directly.
 
 =cut
 
@@ -539,7 +553,9 @@ sub close
    $self->close_when_empty;
 }
 
-=head2 $stream->close_when_empty
+=head2 close_when_empty
+
+   $stream->close_when_empty
 
 If the write buffer is empty, this method calls C<close> on the underlying IO
 handles, and removes the stream from its containing loop. If the write buffer
@@ -563,7 +579,9 @@ sub close_when_empty
    $self->{stream_closing} = 1;
 }
 
-=head2 $stream->close_now
+=head2 close_now
+
+   $stream->close_now
 
 This method immediately closes the underlying IO handles and removes the
 stream from the containing loop. It will not wait to flush the remaining data
@@ -585,9 +603,13 @@ sub close_now
    $self->SUPER::close;
 }
 
-=head2 $eof = $stream->is_read_eof
+=head2 is_read_eof
 
-=head2 $eof = $stream->is_write_eof
+=head2 is_write_eof
+
+   $eof = $stream->is_read_eof
+
+   $eof = $stream->is_write_eof
 
 Returns true after an EOF condition is reported on either the read or the
 write handle, respectively.
@@ -606,7 +628,9 @@ sub is_write_eof
    return $self->{write_eof};
 }
 
-=head2 $stream->write( $data, %params )
+=head2 write
+
+   $stream->write( $data, %params )
 
 This method adds data to the outgoing data queue, or writes it immediately,
 according to the C<autoflush> parameter.
@@ -688,7 +712,9 @@ C<on_flush> continuation will be invoked, if supplied. This can be used to
 obtain a marker, to invoke some code once the output queue has been flushed up
 to this point.
 
-=head2 $stream->write( ... )->get
+=head2 write (scalar)
+
+   $stream->write( ... )->get
 
 If called in non-void context, this method returns a L<Future> which will
 complete (with no value) when the write operation has been flushed. This may
@@ -765,6 +791,12 @@ sub _flush_one_write
 
    die "TODO: head data does not contain a plain string" if ref $head->data;
 
+   if( $IO::Async::Debug::DEBUG > 1 ) {
+      my $data = substr $head->data, 0, $head->writelen;
+      $self->debug_printf( "WRITE len=%d", length $data );
+      IO::Async::Debug::log_hexdump( $data ) if $IO::Async::Debug::DEBUG_FLAGS{Sw};
+   }
+
    my $writer = $self->{writer};
    my $len = $self->$writer( $self->write_handle, $head->data, $head->writelen );
 
@@ -827,7 +859,10 @@ sub write
    if( defined wantarray ) {
       my $orig_on_flush = $on_flush;
       my $orig_on_error = $on_error;
-      $f = $self->loop->new_future;
+
+      my $loop = $self->loop or
+         croak "Cannot ->write data returning a Future to a Stream not in a Loop";
+      $f = $loop->new_future;
       $on_flush = sub {
          $f->done;
          $orig_on_flush->( @_ ) if $orig_on_flush;
@@ -842,8 +877,11 @@ sub write
       };
    }
 
+   my $write_len = $params{write_len};
+   defined $write_len or $write_len = $self->{write_len};
+
    push @{ $self->{writequeue} }, Writer(
-      $data, $params{write_len} // $self->{write_len}, $on_write, $on_flush, $on_error, 0
+      $data, $write_len, $on_write, $on_flush, $on_error, 0
    );
 
    keys %params and croak "Unrecognised keys for ->write - " . join( ", ", keys %params );
@@ -897,6 +935,8 @@ sub _flush_one_read
    my $self = shift;
    my ( $eof ) = @_;
 
+   local $self->{flushing_read} = 1;
+
    my $readqueue = $self->{readqueue};
 
    my $ret;
@@ -969,6 +1009,11 @@ sub _do_read
          return;
       }
 
+      if( $IO::Async::Debug::DEBUG > 1 ) {
+         $self->debug_printf( "READ len=%d", $len );
+         IO::Async::Debug::log_hexdump( $data ) if $IO::Async::Debug::DEBUG_FLAGS{Sr};
+      }
+
       my $eof = $self->{read_eof} = ( $len == 0 );
 
       if( my $encoding = $self->{encoding} ) {
@@ -1014,7 +1059,9 @@ sub on_read_low_watermark
    $self->want_readready_for_read( 1 );
 }
 
-=head2 $stream->push_on_read( $on_read )
+=head2 push_on_read
+
+   $stream->push_on_read( $on_read )
 
 Pushes a new temporary C<on_read> handler to the end of the queue. This queue,
 if non-empty, is used to provide C<on_read> event handling code in preference
@@ -1040,6 +1087,7 @@ sub push_on_read
    push @{ $self->{readqueue} }, Reader( $on_read, $args{future} );
 
    # TODO: Should this always defer?
+   return if $self->{flushing_read};
    1 while length $self->{readbuff} and $self->_flush_one_read( 0 );
 }
 
@@ -1100,9 +1148,13 @@ sub _read_future
    return $f;
 }
 
-=head2 ( $string, $eof ) = $stream->read_atmost( $len )->get
+=head2 read_atmost
 
-=head2 ( $string, $eof ) = $stream->read_exactly( $len )->get
+=head2 read_exactly
+
+   ( $string, $eof ) = $stream->read_atmost( $len )->get
+
+   ( $string, $eof ) = $stream->read_exactly( $len )->get
 
 Completes the C<Future> when the read buffer contains C<$len> or more
 characters of input. C<read_atmost> will also complete after the first
@@ -1142,7 +1194,9 @@ sub read_exactly
    return $f;
 }
 
-=head2 ( $string, $eof ) = $stream->read_until( $end )->get
+=head2 read_until
+
+   ( $string, $eof ) = $stream->read_until( $end )->get
 
 Completes the C<Future> when the read buffer contains a match for C<$end>,
 which may either be a plain string or a compiled C<Regexp> reference. Yields
@@ -1176,7 +1230,9 @@ sub read_until
    return $f;
 }
 
-=head2 ( $string, $eof ) = $stream->read_until_eof->get
+=head2 read_until_eof
+
+   ( $string, $eof ) = $stream->read_until_eof->get
 
 Completes the C<Future> when the stream is eventually closed at EOF, and
 yields all of the data that was available.
@@ -1202,11 +1258,17 @@ sub read_until_eof
 
 =cut
 
-=head2 $stream = IO::Async::Stream->new_for_stdin
+=head2 new_for_stdin
 
-=head2 $stream = IO::Async::Stream->new_for_stdout
+=head2 new_for_stdout
 
-=head2 $stream = IO::Async::Stream->new_for_stdio
+=head2 new_for_stdio
+
+   $stream = IO::Async::Stream->new_for_stdin
+
+   $stream = IO::Async::Stream->new_for_stdout
+
+   $stream = IO::Async::Stream->new_for_stdio
 
 Return a C<IO::Async::Stream> object preconfigured with the correct
 C<read_handle>, C<write_handle> or both.
@@ -1218,7 +1280,9 @@ sub new_for_stdout { shift->new( write_handle => \*STDOUT, @_ ) }
 
 sub new_for_stdio { shift->new( read_handle => \*STDIN, write_handle => \*STDOUT, @_ ) }
 
-=head2 $future = $stream->connect( %args )
+=head2 connect
+
+   $future = $stream->connect( %args )
 
 A convenient wrapper for calling the C<connect> method on the underlying
 L<IO::Async::Loop> object, passing the C<socktype> hint as C<stream> if not
@@ -1232,6 +1296,24 @@ sub connect
    return $self->SUPER::connect( socktype => "stream", @_ );
 }
 
+=head1 DEBUGGING FLAGS
+
+The following flags in C<IO_ASYNC_DEBUG_FLAGS> enable extra logging:
+
+=over 4
+
+=item C<Sr>
+
+Log byte buffers as data is read from a Stream
+
+=item C<Sw>
+
+Log byte buffers as data is written to a Stream
+
+=back
+
+=cut
+
 =head1 EXAMPLES
 
 =head2 A line-based C<on_read> method
@@ -1295,7 +1377,7 @@ C<on_read> method extracts messages in such a protocol.
 
     return 0 unless length $$buffref >= 8; # "N n n" consumes 8 bytes
 
-    my ( $len, $x, $y ) = unpack $$buffref, "N n n";
+    my ( $len, $x, $y ) = unpack "N n n", $$buffref;
 
     return 0 unless length $$buffref >= 8 + $len;
 
